///////////////////////////////////////////////////////////////////////////////
// TemplateException.java - Shows errors in parsing or constructing templates.
// Copyright (C) 2003 Michael A. Ericson (maerics@cs.unm.edu)
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
///////////////////////////////////////////////////////////////////////////////
package com.maerics.template;


/**
 * Reports errors in parsing or constructing a template object.  Whenever
 * possible the line and character number of the character that caused the
 * template error are passed along.
 */
public class TemplateException extends Exception {
  private String _message;
  private String _filename;
  private int _lineNumber;
  private int _charNumber;


  /**
   * Constructs a new template exception with useful diagnostic information.
   * The filename may be null (for example, if the template was parsed from
   * supplied text) and if the line or character numbers are not available you
   * can substitute a negative value for them.
   *
   * @param message the nature of the error that caused this exception.
   * @param filename the name of the file (if available) from which a calling
   *     template object was parsed.
   * @param lineNumber the line of input that caused this error or -1.
   * @param charNumber the character number in the <code>lineNumber</code> that
   *     caused this error or -1.
   */
  public TemplateException(String message,
                           String filename,
			   int lineNumber,
			   int charNumber)
  {
    this._message = message;
    this._filename = filename;
    this._lineNumber = lineNumber;
    this._charNumber = charNumber;
  }


  /**
   * Returns the name of the file that contained the text that generated this
   * exception.  This value can be null if the template was constructed with
   * the "text" attribute rather than the "filename" attribute, for example.
   *
   * @return the name of the file that generated the exception, or null if
   *     there was none.
   */
  public String getFilename() {
    return this._filename;
  }


  /**
   * Returns the line number in the template text that generated this
   * exception.
   *
   * @return the line number in the template text that generated the
   *     exception, or a non-positive value if not known.
   */
  public int getLineNumber() {
    return this._lineNumber;
  }


  /**
   * Returns the character number in the template text that generated this
   * exception.
   *
   * @return the character number in the template text that generated the
   *     exception, or a non-positive value if not known.
   */
  public int getCharacterNumber() {
    return this._charNumber;
  }


  /**
   * Returns a useful message about the nature of the problem during parsing
   * or constructing a <code>Template</code> object.  When ever possible,
   * information about a (1) parsing error, (2) construction error, and (3) the
   * line and character number that caused this error are supplied.
   *
   * @see Template
   */
  public String getMessage() {
    StringBuffer message = new StringBuffer(this._message);

    if ( this._lineNumber > 0 ) {
      message.append( (this._filename == null)
	? new String(" at text:")
	: new String(" at "+this._filename+":"));
      message.append(this._lineNumber);
    }
    if ( this._charNumber > 0 ) {
      message.append(","+this._charNumber);
    }
    return message.toString();
  }
}

